package com.iformall.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

public class RedisCacheUtils {

	private static <T> T parseJson(String json,Class<T> clazz) {
		return JSON.parseObject(json,clazz);
	}
	
	private static <T> List<T> parseListJson(String json,Class<T> clazz) {
		return JSON.parseObject(json,new TypeReference<List<T>>(clazz) {});
	}
	
	private static <T> Set<T> parseSetJson(String json,Class<T> clazz) {
		return JSON.parseObject(json,new TypeReference<Set<T>>(clazz) {});
	}
	
	public static void cache(RedisTemplate<String, Object> template,String key, Object value,long seconds) {
		ValueOperations<String, Object> operations = template.opsForValue();
		if (seconds > 0) {
			operations.set(key, value, seconds,TimeUnit.SECONDS);
		}else {
			operations.set(key, value);
		}
	}
	
	public static <T> T getCacheObject(RedisTemplate<String, Object> template,String key,Class<T> clazz) {
		if (template.hasKey(key)) {
			ValueOperations<String, Object> operations = template.opsForValue();
			Object o = operations.get(key);
			if (o instanceof String) {
				return parseJson((String)o, clazz);
			}else {
				return parseJson(JSON.toJSONString(o), clazz);
			}
			
		}
		return null;
	}
	
	public static <T> List<T> getCacheListObject(RedisTemplate<String, Object> template,String key,Class<T> clazz) {
		if (template.hasKey(key)) {
			ValueOperations<String, Object> operations = template.opsForValue();
			Object o = operations.get(key);
			if (o instanceof String) {
				return parseListJson((String)o, clazz);
			}else {
				return parseListJson(JSON.toJSONString(o), clazz);
			}
		}
		return null;
	}
	
	public static <T> Set<T> getCacheSetObject(RedisTemplate<String, Object> template,String key,Class<T> clazz) {
		if (template.hasKey(key)) {
			ValueOperations<String, Object> operations = template.opsForValue();
			Object o = operations.get(key);
			if (o instanceof String) {
				return parseSetJson((String)o, clazz);
			}else {
				return parseSetJson(JSON.toJSONString(o), clazz);
			}
		}
		return null;
	}	
	
	public static Map getCacheMap(RedisTemplate<String, Object> template,String key) {
		return getCacheObject(template,key,Map.class);
	}	
	
	public static String getCacheString(RedisTemplate<String, Object> template,String key) {
		if (template.hasKey(key)) {
			ValueOperations<String, Object> operations = template.opsForValue();
			Object o = operations.get(key);
			if( o instanceof String) {
				return (String) o;
			}
			return JSON.toJSONString(o);
		}
		return null;
	}
	
	public static Long getCacheLong(RedisTemplate<String, Object> template,String key) {
		if (template.hasKey(key)) {
			ValueOperations<String, Object> operations = template.opsForValue();
			Object o = operations.get(key);
			if (o instanceof String) {
				Long.parseLong((String)o);
			}
			return Long.parseLong(o.toString());
		}
		return null;
	}
	
	public static Integer getCacheInteger(RedisTemplate<String, Object> template,String key) {
		if (template.hasKey(key)) {
			ValueOperations<String, Object> operations = template.opsForValue();
			Object o = operations.get(key);
			if (o instanceof String) {
				return new Integer((String)o);
			}
			return new Integer(o.toString());
		}
		return null;
	}
	
    public static <T> T getFieldObject(Object o,String property,Class<T> clazz) {
    	String json = JSON.toJSONString(o);
    	JSONObject jsonObject = JSON.parseObject(json);
    	Object p = jsonObject.get(property);
    	if (null != p ) {
    		if (p instanceof String) {
    			return parseJson((String)p, clazz);
    		}
    		return parseJson(JSON.toJSONString(p), clazz);
    	}
    	return null;
    }
    
    public static <T> List<T> getFieldListObject(Object o,String property,Class<T> clazz) {
    	String json = JSON.toJSONString(o);
    	JSONObject jsonObject = JSON.parseObject(json);
    	Object p = jsonObject.get(property);
    	if (null != p ) {
    		if (p instanceof String) {
    			return parseListJson((String)p, clazz);
    		}
    		return parseListJson(JSON.toJSONString(p), clazz);
    	}
    	return null;
    }
    
    public static <T> Set<T> getFieldSetObject(Object o,String property,Class<T> clazz) {
    	String json = JSON.toJSONString(o);
    	JSONObject jsonObject = JSON.parseObject(json);
    	Object p = jsonObject.get(property);
    	if (null != p ) {
    		if (p instanceof String) {
    			return parseSetJson((String)p, clazz);
    		}
    		return parseSetJson(JSON.toJSONString(p), clazz);
    	}
    	return null;
    }
    
    public static Map getFieldMapObject(Object o,String property) {
    	return getFieldObject(o,property,Map.class);
    }
	
	public static void removeCache(RedisTemplate<String, Object> template,String key) {
		if (template.hasKey(key)) {
			template.delete(key);
		}
	}
}
